@@ -38,26 +38,28 @@ See the :ref:`admin compatibility page <admin-compat>` for details.
 """
 from __future__ import unicode_literals
 import django
-from django.conf.urls import patterns, url
+from django.conf import settings
+from django.conf.urls import url
 from django.contrib import admin
 from django.contrib.admin.options import csrf_protect_m, BaseModelAdmin, InlineModelAdmin
-from django.contrib.admin.util import get_deleted_objects, unquote
+from django.contrib.admin.utils import get_deleted_objects, quote, unquote
+from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
 from django.core.exceptions import PermissionDenied, ImproperlyConfigured
-from django.core.urlresolvers import reverse
-from django.db import router
+from django.db import router, transaction
 from django.forms import Media
 from django.http import HttpResponseRedirect, Http404, HttpRequest
 from django.shortcuts import render
+from django.urls import reverse
 from django.utils.encoding import iri_to_uri, force_text
-from django.utils.functional import lazy
+from django.utils.functional import cached_property
+from django.utils.html import conditional_escape, escape
 from django.utils.http import urlencode
+from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _, get_language
-from django.utils import six
 from parler import appsettings
 from parler.forms import TranslatableModelForm, TranslatableBaseInlineFormSet
 from parler.managers import TranslatableQuerySet
-from parler.models import TranslatableModel
-from parler.utils.compat import transaction_atomic, add_preserved_filters
+from parler.models import TranslatableModelMixin
 from parler.utils.i18n import get_language_title, is_multilingual_project
 from parler.utils.views import get_language_parameter, get_language_tabs
 from parler.utils.template import select_template_name
@@ -75,11 +77,14 @@ __all__ = (
 )
 
 _language_media = Media(css={
-    'all': ('parler/admin/language_tabs.css',)
+    'all': (
+        'parler/admin/parler_admin.css',
+    )
 })
+
 _language_prepopulated_media = _language_media + Media(js=(
     'admin/js/urlify.js',
-    'admin/js/prepopulate.min.js'
+    'admin/js/prepopulate{}.js'.format('' if settings.DEBUG else '.min'),
 ))
 
 _fakeRequest = HttpRequest()
@@ -95,31 +100,27 @@ class BaseTranslatableAdmin(BaseModelAdmin):
     #: The URL parameter for the language value.
     query_language_key = 'language'
 
-
     @property
     def media(self):
         # Currently, `prepopulated_fields` can't be used because it breaks the admin validation.
         # TODO: as a fix TranslatedFields should become a RelatedField on the shared model (may also support ORM queries)
         # As workaround, declare the fields in get_prepopulated_fields() and we'll provide the admin media automatically.
-        has_prepoplated = len(self.get_prepopulated_fields(_fakeRequest))
+        has_prepopulated = len(self.get_prepopulated_fields(_fakeRequest))
         base_media = super(BaseTranslatableAdmin, self).media
-        if has_prepoplated:
+        if has_prepopulated:
             return base_media + _language_prepopulated_media
         else:
             return base_media + _language_media
 
-
     def _has_translatable_model(self):
         # Allow fallback to regular models when needed.
-        return issubclass(self.model, TranslatableModel)
-
+        return issubclass(self.model, TranslatableModelMixin)
 
     def _language(self, request, obj=None):
         """
         Get the language parameter from the current request.
         """
-        return get_language_parameter(request, self.query_language_key, object=obj)
-
+        return get_language_parameter(request, self.query_language_key)
 
     def get_form_language(self, request, obj=None):
         """
@@ -130,7 +131,6 @@ class BaseTranslatableAdmin(BaseModelAdmin):
         else:
             return self._language(request)
 
-
     def get_queryset_language(self, request):
         """
         Return the language to use in the queryset.
@@ -143,15 +143,11 @@ class BaseTranslatableAdmin(BaseModelAdmin):
             # This is overwritten for the inlines, which follow the primary object.
             return get_language()
 
-
     def get_queryset(self, request):
         """
         Make sure the current language is selected.
         """
-        if django.VERSION >= (1, 6):
-            qs = super(BaseTranslatableAdmin, self).get_queryset(request)
-        else:
-            qs = super(BaseTranslatableAdmin, self).queryset(request)
+        qs = super(BaseTranslatableAdmin, self).get_queryset(request)
 
         if self._has_translatable_model():
             if not isinstance(qs, TranslatableQuerySet):
@@ -164,9 +160,6 @@ class BaseTranslatableAdmin(BaseModelAdmin):
 
         return qs
 
-    # For Django 1.5
-    queryset = get_queryset
-
     def get_language_tabs(self, request, obj, available_languages, css_class=None):
         """
         Determine the language tabs to show.
@@ -175,7 +168,6 @@ class BaseTranslatableAdmin(BaseModelAdmin):
         return get_language_tabs(request, current_language, available_languages, css_class=css_class)
 
 
-
 class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
     """
     Base class for translated admins.
@@ -184,13 +176,14 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
     When using this class with a non-TranslatableModel,
     all operations effectively become a NO-OP.
     """
+    #: Whether the translations should be prefetched when displaying the 'language_column' in the list.
+    prefetch_language_column = True
 
     deletion_not_allowed_template = 'admin/parler/deletion_not_allowed.html'
 
     #: Whether translations of inlines should also be deleted when deleting a translation.
     delete_inline_translations = True
 
-
     @property
     def change_form_template(self):
         """
@@ -205,25 +198,67 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
         else:
             return None # get default admin selection
 
-
     def language_column(self, object):
         """
         The language column which can be included in the ``list_display``.
         """
-        languages = self.get_available_languages(object)
-        languages = [self.get_language_short_title(code) for code in languages]
-        return '<span class="available-languages">{0}</span>'.format(' '.join(languages))
-
-    language_column.allow_tags = True
+        return mark_safe(
+            self._languages_column(object, span_classes='available-languages')
+        )  # span class for backwards compatibility
     language_column.short_description = _("Languages")
 
+    def all_languages_column(self, object):
+        """
+        The language column which can be included in the ``list_display``.
+        It also shows untranslated languages
+        """
+        all_languages = [code for code, __ in settings.LANGUAGES]
+        return mark_safe(
+            self._languages_column(
+                object, all_languages, span_classes='all-languages'
+            )
+        )
+    all_languages_column.short_description = _("Languages")
+
+    def _languages_column(self, object, all_languages=None, span_classes=''):
+        active_languages = self.get_available_languages(object)
+        if all_languages is None:
+            all_languages = active_languages
+
+        current_language = object.get_current_language()
+        buttons = []
+        opts = self.opts
+        for code in (all_languages or active_languages):
+            classes = ['lang-code']
+            if code in active_languages:
+                classes.append('active')
+            else:
+                classes.append('untranslated')
+            if code == current_language:
+                classes.append('current')
+
+            info = opts.app_label, opts.model_name
+            admin_url = reverse('admin:{0}_{1}_change'.format(*info), args=(quote(object.pk),), current_app=self.admin_site.name)
+            buttons.append('<a class="{classes}" href="{href}?language={language_code}">{title}</a>'.format(
+                language_code=code,
+                classes=' '.join(classes),
+                href=escape(admin_url),
+                title=conditional_escape(self.get_language_short_title(code))
+           ))
+        return '<span class="language-buttons {0}">{1}</span>'.format(
+            span_classes,
+            ' '.join(buttons)
+        )
 
     def get_language_short_title(self, language_code):
         """
         Hook for allowing to change the title in the :func:`language_column` of the list_display.
         """
-        return language_code
-
+        # Show language codes in uppercase by default.
+        # This avoids a general text-transform CSS rule,
+        # that might conflict with showing longer titles for a language instead of the code.
+        # (e.g. show "Global" instead of "EN")
+        return language_code.upper()
 
     def get_available_languages(self, obj):
         """
@@ -234,18 +269,29 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
         else:
             return self.model._parler_meta.root_model.objects.none()
 
+    def get_queryset(self, request):
+        qs = super(TranslatableAdmin, self).get_queryset(request)
 
-    def get_object(self, request, object_id):
+        if self.prefetch_language_column:
+            # When the available languages are shown in the listing, prefetch available languages.
+            # This avoids an N-query issue because each row needs the available languages.
+            list_display = self.get_list_display(request)
+            if 'language_column' in list_display or 'all_languages_column' in list_display:
+                qs = qs.prefetch_related(self.model._parler_meta.root_rel_name)
+
+        return qs
+
+    def get_object(self, request, object_id, *args, **kwargs):
         """
         Make sure the object is fetched in the correct language.
         """
-        obj = super(TranslatableAdmin, self).get_object(request, object_id)
+        obj = super(TranslatableAdmin, self).get_object(request, object_id, *args, **kwargs)
+
         if obj is not None and self._has_translatable_model():  # Allow fallback to regular models.
             obj.set_current_language(self._language(request, obj), initialize=True)
 
         return obj
 
-
     def get_form(self, request, obj=None, **kwargs):
         """
         Pass the current language to the form.
@@ -256,7 +302,6 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
 
         return form_class
 
-
     def get_urls(self):
         """
         Add a delete-translation view.
@@ -266,15 +311,12 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
             return urlpatterns
         else:
             opts = self.model._meta
-            info = opts.app_label, opts.model_name if django.VERSION >= (1, 7) else opts.module_name
-
-            return patterns('',
-                url(r'^(.+)/delete-translation/(.+)/$',
-                    self.admin_site.admin_view(self.delete_translation),
-                    name='{0}_{1}_delete_translation'.format(*info)
-                ),
-            ) + urlpatterns
-
+            info = opts.app_label, opts.model_name
+            return [url(
+                r'^(.+)/change/delete-translation/(.+)/$',
+                self.admin_site.admin_view(self.delete_translation),
+                name='{0}_{1}_delete_translation'.format(*info)
+            )] + urlpatterns
 
     def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
         """
@@ -296,52 +338,58 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
             # Otherwise AdminModel.render_change_form will clean the URL
             # and remove the "language" when coming from a filtered object
             # list causing the wrong translation to be changed.
-            form_url = add_preserved_filters({'preserved_filters': urlencode({'language': lang_code}), 'opts': self.model._meta}, form_url)
+
+            params = request.GET.dict()
+            params['language'] = lang_code
+            form_url = add_preserved_filters({
+                'preserved_filters': urlencode(params),
+                'opts': self.model._meta
+            }, form_url)
 
         # django-fluent-pages uses the same technique
         if 'default_change_form_template' not in context:
-            context['default_change_form_template'] = self.get_change_form_base_template()
+            context['default_change_form_template'] = self.default_change_form_template
 
         #context['base_template'] = self.get_change_form_base_template()
         return super(TranslatableAdmin, self).render_change_form(request, context, add, change, form_url, obj)
 
-
     def response_add(self, request, obj, post_url_continue=None):
-        # Minor behavior difference for Django 1.4
-        if post_url_continue is None and django.VERSION < (1,5):
-            post_url_continue = '../%s/'
-
         # Make sure ?language=... is included in the redirects.
         redirect = super(TranslatableAdmin, self).response_add(request, obj, post_url_continue)
         return self._patch_redirect(request, obj, redirect)
 
-
     def response_change(self, request, obj):
         # Make sure ?language=... is included in the redirects.
         redirect = super(TranslatableAdmin, self).response_change(request, obj)
         return self._patch_redirect(request, obj, redirect)
 
-
     def _patch_redirect(self, request, obj, redirect):
-        if redirect.status_code not in (301,302):
+        if redirect.status_code not in (301, 302):
             return redirect  # a 200 response likely.
 
         uri = iri_to_uri(request.path)
         opts = self.model._meta
-        info = (opts.app_label, opts.model_name if django.VERSION >= (1, 7) else opts.module_name)
+        info = opts.app_label, opts.model_name
 
         # Pass ?language=.. to next page.
         language = request.GET.get(self.query_language_key)
         if language:
-            continue_urls = (uri, "../add/", reverse('admin:{0}_{1}_add'.format(*info)))
-            if redirect['Location'] in continue_urls and self.query_language_key in request.GET:
+            continue_urls = (
+                uri,
+                "../add/",
+                reverse('admin:{0}_{1}_add'.format(*info), current_app=self.admin_site.name),
+                "../change/",
+                reverse('admin:{0}_{1}_change'.format(*info), args=[obj.pk,], current_app=self.admin_site.name),
+            )
+            redirect_parts = redirect['Location'].split('?')
+            if redirect_parts[0] in continue_urls and self.query_language_key in request.GET:
                 # "Save and add another" / "Save and continue" URLs
-                redirect['Location'] += "?{0}={1}".format(self.query_language_key, language)
+                delimiter = '&' if len(redirect_parts) > 1 else '?'
+                redirect['Location'] += "{0}{1}={2}".format(delimiter, self.query_language_key, language)
         return redirect
 
-
     @csrf_protect_m
-    @transaction_atomic
+    @transaction.atomic
     def delete_translation(self, request, object_id, language_code):
         """
         The 'delete translation' admin view for this model.
@@ -351,6 +399,9 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
 
         # Get object and translation
         shared_obj = self.get_object(request, unquote(object_id))
+        if shared_obj is None:
+            raise Http404
+
         shared_obj.set_current_language(language_code)
         try:
             translation = root_model.objects.get(master=shared_obj, language_code=language_code)
@@ -360,7 +411,7 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
         if not self.has_delete_permission(request, translation):
             raise PermissionDenied
 
-        if self.get_available_languages(shared_obj).count() <= 1:
+        if len(self.get_available_languages(shared_obj)) <= 1:
             return self.deletion_not_allowed(request, translation, language_code)
 
         # Populate deleted_objects, a data structure of all related objects that
@@ -378,12 +429,17 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
 
         # Extend deleted objects with the inlines.
         for qs in self.get_translation_objects(request, translation.language_code, obj=shared_obj, inlines=self.delete_inline_translations):
-            if isinstance(qs, (list,tuple)):
+            if isinstance(qs, (list, tuple)):
                 qs_opts = qs[0]._meta
             else:
                 qs_opts = qs.model._meta
 
-            (del2, perms2, protected2) = get_deleted_objects(qs, qs_opts, request.user, self.admin_site, using)
+            if django.VERSION >= (2, 1):
+                deleted_result = get_deleted_objects(qs, request, self.admin_site)
+            else:
+                deleted_result = get_deleted_objects(qs, qs_opts, request.user, self.admin_site, using)
+            (del2, model_counts, perms2, protected2) = deleted_result
+
             deleted_objects += del2
             perms_needed = perms_needed or perms2
             protected += protected2
@@ -400,9 +456,10 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
             ))
 
             if self.has_change_permission(request, None):
-                return HttpResponseRedirect(reverse('admin:{0}_{1}_changelist'.format(opts.app_label, opts.model_name if django.VERSION >= (1, 7) else opts.module_name)))
+                info = opts.app_label, opts.model_name
+                return HttpResponseRedirect(reverse('admin:{0}_{1}_change'.format(*info), args=(object_id,), current_app=self.admin_site.name))
             else:
-                return HttpResponseRedirect(reverse('admin:index'))
+                return HttpResponseRedirect(reverse('admin:index', current_app=self.admin_site.name))
 
         object_name = _('{0} Translation').format(force_text(opts.verbose_name))
         if perms_needed or protected:
@@ -421,13 +478,20 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
             "app_label": opts.app_label,
         }
 
+        # Small hack for django-polymorphic-tree.
+        # This makes sure the breadcrumb renders correctly,
+        # and avoids errors when the child model is not registered in the admin.
+        if hasattr(self, 'base_model'):
+            context.update({
+                'base_opts': self.base_model._meta,
+            })
+
         return render(request, self.delete_confirmation_template or [
             "admin/%s/%s/delete_confirmation.html" % (opts.app_label, opts.object_name.lower()),
             "admin/%s/delete_confirmation.html" % opts.app_label,
             "admin/delete_confirmation.html"
         ], context)
 
-
     def deletion_not_allowed(self, request, obj, language_code):
         """
         Deletion-not-allowed view.
@@ -443,7 +507,6 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
         }
         return render(request, self.deletion_not_allowed_template, context)
 
-
     def delete_model_translation(self, request, translation):
         """
         Hook for deleting a translation.
@@ -452,7 +515,7 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
         """
         master = translation.master
         for qs in self.get_translation_objects(request, translation.language_code, obj=master, inlines=self.delete_inline_translations):
-            if isinstance(qs, (tuple,list)):
+            if isinstance(qs, (tuple, list)):
                 # The objects are deleted one by one.
                 # This triggers the post_delete signals and such.
                 for obj in qs:
@@ -462,7 +525,6 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
                 # This doesn't trigger signals, just like the regular
                 qs.delete()
 
-
     def get_translation_objects(self, request, language_code, obj=None, inlines=True):
         """
         Return all objects that should be deleted when a translation is deleted.
@@ -482,19 +544,13 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
             for inline, qs in self._get_inline_translations(request, language_code, obj=obj):
                 yield qs
 
-
     def _get_inline_translations(self, request, language_code, obj=None):
         """
         Fetch the inline translations
         """
-        # django 1.4 do not accept the obj parameter
-        if django.VERSION < (1, 5):
-            inline_instances = self.get_inline_instances(request)
-        else:
-            inline_instances = self.get_inline_instances(request, obj=obj)
-
+        inline_instances = self.get_inline_instances(request, obj=obj)
         for inline in inline_instances:
-            if issubclass(inline.model, TranslatableModel):
+            if issubclass(inline.model, TranslatableModelMixin):
                 # leverage inlineformset_factory() to find the ForeignKey.
                 # This also resolves the fk_name if it's set.
                 fk = inline.get_formset(request, obj).fk
@@ -512,23 +568,20 @@ class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
 
                     yield inline, qs
 
-
-    def get_change_form_base_template(self):
+    @cached_property
+    def default_change_form_template(self):
         """
         Determine what the actual `change_form_template` should be.
         """
         opts = self.model._meta
         app_label = opts.app_label
-        return _lazy_select_template_name((
+        return select_template_name((
             "admin/{0}/{1}/change_form.html".format(app_label, opts.object_name.lower()),
             "admin/{0}/change_form.html".format(app_label),
             "admin/change_form.html"
         ))
 
 
-_lazy_select_template_name = lazy(select_template_name, six.text_type)
-
-
 class TranslatableInlineModelAdmin(BaseTranslatableAdmin, InlineModelAdmin):
     """
     Base class for inline models.
@@ -547,7 +600,7 @@ class TranslatableInlineModelAdmin(BaseTranslatableAdmin, InlineModelAdmin):
 
     def _has_translatable_parent_model(self):
         # Allow fallback to regular models when needed.
-        return issubclass(self.parent_model, TranslatableModel)
+        return issubclass(self.parent_model, TranslatableModelMixin)
 
     def get_queryset_language(self, request):
         if not is_multilingual_project():
@@ -646,6 +699,7 @@ class SortedRelatedFieldListFilter(admin.RelatedFieldListFilter):
                 ('related_field_name', SortedRelatedFieldListFilter),
             )
     """
+
     def __init__(self, *args, **kwargs):
         super(SortedRelatedFieldListFilter, self).__init__(*args, **kwargs)
         self.lookup_choices = sorted(self.lookup_choices, key=lambda a: a[1].lower())
