@@ -10,9 +10,52 @@ It's also useful for abstract models; add a :class:`TranslatedField` to
 indicate that the derived model is expected to provide that translatable field.
 """
 from __future__ import unicode_literals
-from django.core.exceptions import FieldError
-from django.db.models.fields import Field
-from django.forms.forms import pretty_name
+
+from django.core.exceptions import ImproperlyConfigured
+from django.db import models
+from django.db.models.fields.related_descriptors import ForwardManyToOneDescriptor
+from django.forms.utils import pretty_name
+
+
+def _validate_master(new_class):
+    """
+    Check whether the 'master' field on a TranslatedFieldsModel is correctly configured.
+    """
+    if not new_class.master or not isinstance(new_class.master, ForwardManyToOneDescriptor):
+        raise ImproperlyConfigured("{0}.master should be a ForeignKey to the shared table.".format(new_class.__name__))
+
+    remote_field = new_class.master.field.remote_field
+    shared_model = remote_field.model
+
+    # Skip checks in migration
+    if shared_model.__module__ == '__fake__':
+        return shared_model
+
+    try:
+        meta = shared_model._parler_meta
+    except AttributeError:
+        raise TypeError("Translatable model {} does not appear to inherit from TranslatableModel".format(shared_model))
+
+    if meta is not None:
+        if meta._has_translations_model(new_class):
+            raise ImproperlyConfigured("The model '{0}' already has an associated translation table!".format(shared_model.__name__))
+        if meta._has_translations_field(remote_field.related_name):
+            raise ImproperlyConfigured("The model '{0}' already has an associated translation field named '{1}'!".format(shared_model.__name__, remote_field.related_name))
+
+    return shared_model
+
+
+class TranslationsForeignKey(models.ForeignKey):
+    """
+    Foreign Key field that adds translations to the `to` model.
+    """
+    def contribute_to_related_class(self, cls, related):
+        from parler.models import TranslatedFieldsModelMixin
+
+        super(TranslationsForeignKey, self).contribute_to_related_class(cls, related)
+        _validate_master(self.model)
+        if issubclass(self.model, TranslatedFieldsModelMixin):
+            self.model.contribute_translations(cls)
 
 
 # TODO: inherit RelatedField?
@@ -44,13 +87,14 @@ class TranslatedField(object):
             title = models.CharField("Title", max_length=200)
             slug = models.SlugField("Slug")
     """
+
     def __init__(self, any_language=False):
         self.model = None
         self.name = None
         self.any_language = any_language
         self._meta = None
 
-    def contribute_to_class(self, cls, name):
+    def contribute_to_class(self, cls, name, **kwargs):
         #super(TranslatedField, self).contribute_to_class(cls, name)
         self.model = cls
         self.name = name
@@ -71,6 +115,7 @@ class TranslatedFieldDescriptor(object):
 
     This attribute proxies all get/set calls to the translated model.
     """
+
     def __init__(self, field):
         """
         :type field: TranslatedField
@@ -123,7 +168,7 @@ class TranslatedFieldDescriptor(object):
         """
         Ensure that the admin ``list_display`` renders the correct verbose name for translated fields.
 
-        The :func:`~django.contrib.admin.util.label_for_field` function
+        The :func:`~django.contrib.admin.utils.label_for_field` function
         uses :func:`~django.db.models.Options.get_field_by_name` to find the find and ``verbose_name``.
         However, for translated fields, this option does not exist,
         hence it falls back to reading the attribute and trying ``short_description``.
@@ -136,16 +181,18 @@ class TranslatedFieldDescriptor(object):
             # Fallback to what the admin label_for_field() would have done otherwise.
             return pretty_name(self.field.name)
 
-        return translations_model._meta.get_field_by_name(self.field.name)[0].verbose_name
+        field = translations_model._meta.get_field(self.field.name)
+        return field.verbose_name
 
 
 class LanguageCodeDescriptor(object):
     """
     This is the property to access the ``language_code`` in the ``TranslatableModel``.
     """
+
     def __get__(self, instance, instance_type=None):
         if not instance:
-            raise AttributeError("language_code must be accessed via instance")
+            return self
 
         return instance._current_language
 
@@ -154,14 +201,3 @@ class LanguageCodeDescriptor(object):
 
     def __delete__(self, instance):
         raise AttributeError("The 'language_code' attribute cannot be deleted!")
-
-
-try:
-    from south.modelsinspector import add_ignored_fields
-except ImportError:
-    pass
-else:
-    _name_re = "^" + __name__.replace(".", "\.")
-    add_ignored_fields((
-        _name_re + "\.TranslatedField",
-    ))
