@@ -2,11 +2,11 @@
 Custom generic managers
 """
 import django
+import six
 from django.core.exceptions import ImproperlyConfigured
 from django.db import models
 from django.db.models.query import QuerySet
 from django.utils.translation import get_language
-from django.utils import six
 from parler import appsettings
 from parler.utils import get_active_language_choices
 
@@ -21,14 +21,48 @@ class TranslatableQuerySet(QuerySet):
 
     def __init__(self, *args, **kwargs):
         super(TranslatableQuerySet, self).__init__(*args, **kwargs)
-        self._language = []
+        self._language = None
 
-
-    def _clone(self, klass=None, setup=False, **kw):
-        c = super(TranslatableQuerySet, self)._clone(klass, setup, **kw)
+    def _clone(self):
+        c = super(TranslatableQuerySet, self)._clone()
         c._language = self._language
         return c
 
+    def create(self, **kwargs):
+        # Pass language setting to the object, as people start assuming things
+        # like .language('xx').create(..) which is a nice API after all.
+        if self._language:
+            kwargs['_current_language'] = self._language
+        return super(TranslatableQuerySet, self).create(**kwargs)
+
+    def _fetch_all(self):
+        # Make sure the current language is assigned when Django fetches the data.
+        # This low-level method is overwritten as that works better across Django versions.
+        # Alternatives includes hacking the _iterable_class, which breaks django-polymorphic
+        super(TranslatableQuerySet, self)._fetch_all()
+        if self._language is not None and self._result_cache and isinstance(self._result_cache[0], models.Model):
+            for obj in self._result_cache:
+                obj.set_current_language(self._language)
+
+    def _extract_model_params(self, defaults, **kwargs):
+        # default implementation in Django>=1.11 doesn't allow non-field attributes,
+        # so process them manually
+        translated_defaults = {}
+        if defaults:
+            for field in self.model._parler_meta.get_all_fields():
+                try:
+                    translated_defaults[field] = defaults.pop(field)
+                except KeyError:
+                    pass
+
+        if django.VERSION < (2, 2):
+            lookup, params = super(TranslatableQuerySet, self)._extract_model_params(defaults, **kwargs)
+            params.update(translated_defaults)
+            return lookup, params
+        else:
+            params = super(TranslatableQuerySet, self)._extract_model_params(defaults, **kwargs)
+            params.update(translated_defaults)
+            return params
 
     def language(self, language_code=None):
         """
@@ -40,7 +74,6 @@ class TranslatableQuerySet(QuerySet):
         self._language = language_code
         return self
 
-
     def translated(self, *language_codes, **translated_fields):
         """
         Only return translated objects which of the given languages.
@@ -77,7 +110,6 @@ class TranslatableQuerySet(QuerySet):
             filters[relname + '__language_code__in'] = language_codes
             return self.filter(**filters).distinct()
 
-
     def active_translations(self, language_code=None, **translated_fields):
         """
         Only return objects which are translated, or have a fallback that should be displayed.
@@ -93,71 +125,16 @@ class TranslatableQuerySet(QuerySet):
         return self.translated(*language_codes, **translated_fields)
 
 
-    def iterator(self):
-        """
-        Overwritten iterator which will set the current language before returning the object.
-        """
-        # Based on django-queryset-transform.
-        # This object however, operates on a per-object instance
-        # without breaking the result generators
-        base_iterator = super(TranslatableQuerySet, self).iterator()
-        for obj in base_iterator:
-            # Apply the language setting.
-            if self._language:
-                obj.set_current_language(self._language)
-
-            yield obj
-
-
-class TranslatableManager(models.Manager):
+class TranslatableManager(models.Manager.from_queryset(TranslatableQuerySet)):
     """
     The manager class which ensures the enhanced TranslatableQuerySet object is used.
     """
-    queryset_class = TranslatableQuerySet
 
     def get_queryset(self):
-        if not issubclass(self.queryset_class, TranslatableQuerySet):
-            raise ImproperlyConfigured("{0}.queryset_class does not inherit from TranslatableQuerySet".format(self.__class__.__name__))
-        return self.queryset_class(self.model, using=self._db)
-
-    # For Django 1.5
-    # Leave for Django 1.6/1.7, so backwards compatibility can be fixed.
-    # It will be removed in Django 1.8, so remove it here too to avoid false promises.
-    if django.VERSION < (1,8):
-        get_query_set = get_queryset
-
-    def language(self, language_code=None):
-        """
-        Set the language code to assign to objects retrieved using this Manager.
-        """
-        return self.get_queryset().language(language_code)
-
-    def translated(self, *language_codes, **translated_fields):
-        """
-        Only return objects which are translated in the given languages.
-
-        NOTE: due to Django `ORM limitations <https://docs.djangoproject.com/en/dev/topics/db/queries/#spanning-multi-valued-relationships>`_,
-        this method can't be combined with other filters that access the translated fields. As such, query the fields in one filter:
-
-        .. code-block:: python
-
-            qs.translated('en', name="Cheese Omelette")
-
-        This will query the translated model for the ``name`` field.
-        """
-        return self.get_queryset().translated(*language_codes, **translated_fields)
-
-    def active_translations(self, language_code=None, **translated_fields):
-        """
-        Only return objects which are translated, or have a fallback that should be displayed.
-
-        Typically that's the currently active language and fallback language.
-        This should be combined with ``.distinct()``.
-
-        When ``hide_untranslated = True``, only the currently active language will be returned.
-        """
-        return self.get_queryset().active_translations(language_code, **translated_fields)
-
+        qs = super(TranslatableManager, self).get_queryset()
+        if not isinstance(qs, TranslatableQuerySet):
+            raise ImproperlyConfigured("{0}._queryset_class does not inherit from TranslatableQuerySet".format(self.__class__.__name__))
+        return qs
 
 # Export the names in django-hvad style too:
 TranslationQueryset = TranslatableQuerySet
