@@ -1,10 +1,11 @@
+import six
 from django.template import TemplateDoesNotExist
-from django.template.loader import find_template
+from django.template.loader import get_template
 
 _cached_name_lookups = {}
 
 
-def select_template_name(template_name_list):
+def select_template_name(template_name_list, using=None):
     """
     Given a list of template names, find the first one that exists.
     """
@@ -17,12 +18,12 @@ def select_template_name(template_name_list):
         # Find which template of the template_names is selected by the Django loader.
         for template_name in template_name_list:
             try:
-                find_template(template_name)
+                get_template(template_name, using=using)
             except TemplateDoesNotExist:
                 continue
             else:
-                template_name = unicode(template_name)  # consistent value for lazy() function.
+                template_name = six.text_type(template_name)  # consistent value for lazy() function.
                 _cached_name_lookups[template_name_list] = template_name
                 return template_name
 
-        return None
\ No newline at end of file
+        return None
