@@ -14,20 +14,20 @@ The following views are available:
 * :class:`TranslatableUpdateView` - The :class:`~django.views.generic.edit.UpdateView` with :class:`TranslatableModelFormMixin` support.
 """
 from __future__ import unicode_literals
-import django
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
-from django.core.urlresolvers import reverse
 from django.forms.models import modelform_factory
 from django.http import Http404, HttpResponsePermanentRedirect
+from django.urls import reverse
 from django.utils import translation
 from django.views import generic
 from django.views.generic.edit import ModelFormMixin
 from parler.forms import TranslatableModelForm
-from parler.models import TranslatableModel
+from parler.models import TranslatableModelMixin
 from parler.utils import get_active_language_choices
 from parler.utils.context import switch_language
 from parler.utils.views import get_language_parameter, get_language_tabs
 
+
 __all__ = (
     'ViewUrlMixin',
     'TranslatableSlugMixin',
@@ -70,7 +70,6 @@ class ViewUrlMixin(object):
     #: should correspond with the view name in the URLConf.
     view_url_name = None
 
-
     def get_view_url(self):
         """
         This method is used by the ``get_translated_url`` template tag.
@@ -91,16 +90,6 @@ class ViewUrlMixin(object):
         return reverse(self.view_url_name, args=self.args, kwargs=self.kwargs)
 
 
-    if django.VERSION < (1,5):
-        # The `get_translated_url` tag relies on the fact that the template can access the view again.
-        # This was not possible until Django 1.5, so provide the `ContextMixin` logic for earlier Django versions.
-
-        def get_context_data(self, **kwargs):
-            if 'view' not in kwargs:
-                kwargs['view'] = self
-            return kwargs
-
-
 class TranslatableSlugMixin(object):
     """
     An enhancement for the :class:`~django.views.generic.DetailView` to deal with translated slugs.
@@ -119,6 +108,7 @@ class TranslatableSlugMixin(object):
             model = Article
             template_name = 'article/details.html'
     """
+
     def get_translated_filters(self, slug):
         """
         Allow passing other filters for translated fields.
@@ -199,12 +189,12 @@ class FallbackLanguageResolved(Exception):
     An object was resolved in the fallback language, while it could be in the normal language.
     This exception is used internally to control code flow.
     """
+
     def __init__(self, object, correct_language):
         self.object = object
         self.correct_language = correct_language
 
 
-
 class LanguageChoiceMixin(object):
     """
     Mixin to add language selection support to class based views, particularly create and update views.
@@ -212,23 +202,20 @@ class LanguageChoiceMixin(object):
     """
     query_language_key = 'language'
 
-
     def get_object(self, queryset=None):
         """
         Assign the language for the retrieved object.
         """
         object = super(LanguageChoiceMixin, self).get_object(queryset)
-        if isinstance(object, TranslatableModel):
-            object.set_current_language(self._language(object), initialize=True)
+        if isinstance(object, TranslatableModelMixin):
+            object.set_current_language(self.get_language(), initialize=True)
         return object
 
-
-    def _language(self, object=None):
+    def get_language(self):
         """
         Get the language parameter from the current request.
         """
-        return get_language_parameter(self.request, self.query_language_key, object=object, default=self.get_default_language(object=object))
-
+        return get_language_parameter(self.request, self.query_language_key, default=self.get_default_language(object=object))
 
     def get_default_language(self, object=None):
         """
@@ -238,23 +225,21 @@ class LanguageChoiceMixin(object):
         # Some users may want to override this, to return get_language()
         return None
 
-
     def get_current_language(self):
         """
         Return the current language for the currently displayed object fields.
+        This reads ``self.object.get_current_language()`` and falls back to :func:`get_language`.
         """
         if self.object is not None:
             return self.object.get_current_language()
         else:
-            return self._language()
-
+            return self.get_language()
 
     def get_context_data(self, **kwargs):
         context = super(LanguageChoiceMixin, self).get_context_data(**kwargs)
         context['language_tabs'] = self.get_language_tabs()
         return context
 
-
     def get_language_tabs(self):
         """
         Determine the language tabs to show.
@@ -268,10 +253,6 @@ class LanguageChoiceMixin(object):
         return get_language_tabs(self.request, current_language, available_languages)
 
 
-# Backwards compatibility
-TranslatableSingleObjectMixin = LanguageChoiceMixin
-
-
 class TranslatableModelFormMixin(LanguageChoiceMixin):
     """
     Mixin to add translation support to class based views.
@@ -290,7 +271,9 @@ class TranslatableModelFormMixin(LanguageChoiceMixin):
         Return a ``TranslatableModelForm`` by default if no form_class is set.
         """
         super_method = super(TranslatableModelFormMixin, self).get_form_class
-        if not (super_method.__func__ is ModelFormMixin.get_form_class.__func__):
+        # no "__func__" on the class level function in python 3
+        default_method = getattr(ModelFormMixin.get_form_class, '__func__', ModelFormMixin.get_form_class)
+        if not (super_method.__func__ is default_method):
             # Don't get in your way, if you've overwritten stuff.
             return super_method()
         else:
@@ -299,8 +282,11 @@ class TranslatableModelFormMixin(LanguageChoiceMixin):
                 return self.form_class
             else:
                 model = _get_view_model(self)
-                return modelform_factory(model, form=TranslatableModelForm)
-
+                if self.fields:
+                    fields = self.fields
+                    return modelform_factory(model, form=TranslatableModelForm, fields=fields)
+                else:
+                    return modelform_factory(model, form=TranslatableModelForm)
 
     def get_form_kwargs(self):
         """
@@ -312,9 +298,10 @@ class TranslatableModelFormMixin(LanguageChoiceMixin):
         kwargs['_current_language'] = self.get_form_language()
         return kwargs
 
-
     # Backwards compatibility
-    get_form_language = LanguageChoiceMixin.get_current_language
+    # Make sure overriding get_current_language() affects get_form_language() too.
+    def get_form_language(self):
+        return self.get_current_language()
 
 
 # For the lazy ones:
@@ -336,7 +323,6 @@ class TranslatableUpdateView(TranslatableModelFormMixin, generic.UpdateView):
     pass
 
 
-
 def _get_view_model(self):
     if self.model is not None:
         # If a model has been explicitly provided, use it
@@ -347,3 +333,7 @@ def _get_view_model(self):
     else:
         # Try to get a queryset and extract the model class from that
         return self.get_queryset().model
+
+
+# Backwards compatibility
+TranslatableSingleObjectMixin = LanguageChoiceMixin
